@@ -10,7 +10,7 @@ use Tk::widgets qw/ JPEG LabEntry Pane PNG /;
 use base qw/ Tk::Derived Tk::Pane /;
 use vars qw/ $VERSION $err $info $CORNER $haveAnimation/;
 
-$VERSION = '2.01';
+$VERSION = '2.1';
 $haveAnimation = 0;
 eval 'use Tk::widgets qw/ Animation /; $haveAnimation = 1; 1';
 
@@ -152,7 +152,7 @@ sub button1 {  #LEGACY Tk::Thumbnail DEFAULT MOUSE-BUTTON 1 CALLBACK TO DISPLAY
 	} elsif ( $file =~ /\.xpm$/io) {  #BAD XPMs CAN CRASH PERL, SO JUST USE WHAT ALREADY LOADED!:
 		$p = $extphoto  if ($extphoto);
 		$can_del = 0;
-	} else {
+	} elsif ( $file =~ /\.(?:gif|jpg|jpeg|png)$/i) {
 		Tk::catch { $p = $tl->Photo( -file => $file ); };
 		if ($@ && $extphoto) {
 			$p = $extphoto;
@@ -160,6 +160,13 @@ sub button1 {  #LEGACY Tk::Thumbnail DEFAULT MOUSE-BUTTON 1 CALLBACK TO DISPLAY
 		} else {
 			$can_del = 1;
 		}
+	} else {
+		if ($extphoto) {
+			$p = $extphoto;
+			$can_del = 0;
+		} else {
+			$can_del = 1;
+		}
 	}
 	$tl->protocol( 'WM_DELETE_WINDOW' => sub {
 		$p->delete if $can_del;
@@ -327,6 +334,7 @@ sub render {
 	my( $self ) = @_;
 
 	$self->clear;		# clear Table
+#!!	delete $self->{'descendants'};
 
 	my $pxx = $self->cget( '-iwidth' );  # thumbnail pixel width
 	my $pxy = $self->cget( '-iheight' ); # thumbnail pixel height
@@ -397,12 +405,12 @@ sub render {
 						$imgdata[1] =~ s#\/\*#\\\*#gs;
 						Tk::catch { $photo = $self->Photo(-data => ($imgdata[0].'{'.$imgdata[1]), -format => 'xpm') };
 					}
-				} else {  #FOR NON-XPM IMAGES, LOAD THE NORMAL WAY:
+				} elsif ($ext =~ /^(?:gif|jpg|jpeg|png)$/o) {  #FOR NON-XPM IMAGES, LOAD THE NORMAL WAY:
 					Tk::catch { $photo = $self->Photo( -file => $i ) };
-				}
-				unless ( $@ || !$haveAnimation ) {  #GOOD IMAGE, SEE IF WE'RE AN ANIMATED GIF:
-					Tk::catch { $photo = $self->Animation( -file => $i, -format => 'gif' ) };
-					$animated = 1 unless $@;
+					unless ( $@ || !$haveAnimation ) {  #GOOD IMAGE, SEE IF WE'RE AN ANIMATED GIF:
+						Tk::catch { $photo = $self->Animation( -file => $i, -format => 'gif' ) };
+						$animated = 1 unless $@;
+					}
 				}
 			}
 
@@ -411,6 +419,18 @@ sub render {
 				{
 					if ( UNIVERSAL::isa( $extimg->{$ext}, 'Tk::Photo' ) ) {
 						$photo = $extimg->{$ext};
+					} elsif ($extimg->{$ext} =~ /\.xpm$/io) {  #WORK AROUND STUPID PERL BUG THAT *SEGFAULTS* IF XPM PIXMAP DATA CONTAINS "/*"?!:
+						if (open IN, $extimg->{$ext}) {
+							my $img = '';
+							while (<IN>) {
+								$img .= $_;
+							}
+							close IN;
+							my @imgdata = split(/\{/o, $img, 2);
+							$img = '';
+							$imgdata[1] =~ s#\/\*#\\\*#gs;
+							Tk::catch { $photo = $self->Photo(-data => ($imgdata[0].'{'.$imgdata[1]), -format => 'xpm') };
+						}
 					} else {
 						Tk::catch { $photo = $self->Photo( -file => $extimg->{$ext} ) };
 					}
@@ -460,6 +480,28 @@ sub render {
 			$b->bind('<Button-6>', sub { $self->xview(scroll => -5, 'units')});
 			$b->bind('<Button-7>', sub { $self->xview(scroll => 5, 'units')});
 			$b->bind('<B1-Motion>', sub { $self->Motion(Ev('index',Ev('@')))});
+			$b->bind('<Shift-ButtonPress-1>', sub {
+				my $clickedon = $self->index('mouse');
+				my $anchor = $self->{'anchor'};
+				if (defined($anchor) && ($anchor >= 0 && $anchor <= $self->index('end'))) {
+					if ($self->{'deselecting'}) {
+						$self->selectionClear($anchor, $clickedon);
+					} else {
+						$self->selectionSet($anchor, $clickedon);
+					}
+				} else {
+					if ($self->selectionIncludes($clickedon))  #TOGGLE SELECT-STATUS OF ENTRY CLICKED ON:
+					{
+						$self->selectionClear($clickedon);
+					}
+					else
+					{
+						$self->selectionSet($clickedon);
+					}
+				}
+				$self->{'anchor'} = $clickedon;
+				$self->{'_shifted'} = 1;
+			});
 			$b->bind('<ButtonPress-1>', sub {   #NEEDED FOR "MOTION-DRAG SELECT TO WORK:
 					my $clickedon = $self->index('mouse');
 					$self->activate($clickedon);
@@ -482,8 +524,15 @@ sub render {
 			$photo->delete unless UNIVERSAL::isa( $i, 'Tk::Photo' ) or $photo == $default;
 
 			#BIND THE LEGACY CALLBACK (UNLESS -command => undef):
-			$b->bind('<ButtonRelease-1>' => [ $self => 'Callback', '-command', $self, $indx ])
-					if (defined $self->cget('-command'));  #WE NOW JUST PASS THE INDEX, ${$self->{'data'}}[$indx] HAS ALL THE DATA!
+			if (defined $self->cget('-command')) {  #WE NOW JUST PASS THE INDEX, ${$self->{'data'}}[$indx] HAS ALL THE DATA!
+				$b->bind('<ButtonRelease-1>' => [ $self => 'Callback', '-command', $self, $indx ])
+			} else {
+				$b->bind('<ButtonRelease-1>', sub {
+					$self->selectionToggle($self->index('mouse'))  unless (defined($self->{'_shifted'}) && $self->{'_shifted'});
+					$self->activate($self->index('mouse'));
+					$self->{'_shifted'} = 0;
+				});
+			}
 
 			#BIND ALL THE bindImages SEQUENCES TO EACH IMAGE SUBWIDGET (CAN OVERRIDE LEGACY CALLBACK BINDING ABOVE!):
 			foreach my $bindkey (keys %{$binds}) {
@@ -508,6 +557,36 @@ sub render {
 		$self->{'frame'}->parent->bind('<Up>', sub { my $self = shift->parent; my $i = $self->index('active'); $self->activate($i-$self->{'cols'}); });
 		$self->{'frame'}->parent->bind('<Down>', sub { my $self = shift->parent; my $i = $self->index('active'); $self->activate($i+$self->{'cols'}); });
 		$self->{'frame'}->parent->bind('<ButtonPress-1>', sub { my $self = shift; $self->focus(); })  if ($takefocus > 1);
+		$self->{'frame'}->parent->bind('<Shift-space>', sub {
+				my $self = shift->parent;
+				my $clickedon = $self->index('active');
+				my $anchor = $self->{'anchor'};
+				if (defined($anchor) && ($anchor >= 0 && $anchor <= $self->index('end'))) {
+					if ($self->{'deselecting'}) {
+						$self->selectionClear($anchor, $clickedon);
+					} else {
+						$self->selectionSet($anchor, $clickedon);
+					}
+				} else {
+					if ($self->selectionIncludes($clickedon))  #TOGGLE SELECT-STATUS OF ENTRY CLICKED ON:
+					{
+						$self->selectionClear($clickedon);
+					}
+					else
+					{
+						$self->selectionSet($clickedon);
+					}
+				}
+				$self->{'anchor'} = $clickedon;
+		});
+		$self->{'frame'}->parent->bind('<space>', sub {
+				my $self = shift->parent;
+				my $clickedon = $self->index('active');
+				#$self->focus();
+				$self->{'anchor'} = $self->{'prev'} = $clickedon;
+				$self->{'deselecting'} = $self->isSelected($clickedon);
+				$self->selectionToggle($clickedon);
+		});
 		$self->{'frame'}->configure('-takefocus' => (defined($takefocus) && !$takefocus) ? 0 : 1);
 		$self->{'frame'}->bind('<Home>',  sub { shift->parent->xview('moveto' =>  0) });
 		$self->{'frame'}->bind('<End>',   sub { shift->parent->xview('moveto' =>  1) });
@@ -540,6 +619,12 @@ sub bindRows {   #SYNONYM FOR bindImages(), NAMED FOR COMPAT. W/Tk::HMListbox IN
 	my $subwidget = $w;
 	return (keys %{$w->{'_binds'}})  unless (defined $sequence);
 
+#	unless (defined $callback) {
+#		$callback = $w->{'_bindings'}->{$subwidget}->{$sequence};
+#		$callback = '' unless defined $callback;
+#		return $callback;
+#	}
+
 	if ($callback eq '') {
 		delete $w->{'_binds'}->{$sequence};
 		return '';
@@ -563,6 +648,7 @@ sub clear {
 			$c->gridForget;
 			$c->destroy;
 		}
+		delete $self->{'descendants'};
 	}
 	%{$self->{'selected'}} = ();
 	@{$self->{'data'}} = ();
@@ -720,6 +806,18 @@ sub isSelected
 	return ${$self->{'selected'}}{$self->get($indx)} ? 1 : 0;
 }
 
+sub selectionIncludes
+{
+	return shift->isSelected(@_);
+}
+
+sub selectionAnchor
+{
+	my ($self, $indx) = @_;
+
+	$self->{'anchor'} = $indx;
+}
+
 sub selectionToggle
 {
 	my ($self, $indx) = @_;
@@ -737,20 +835,21 @@ sub selectionToggle
 
 sub selectionClear
 {
-	my ($self, $indx, $indx2) = @_;
+	my $self = shift;
+	my @args = @_;
 
-	$indx = $#{$self->{'descendants'}}  if ($indx =~ /^end$/io);
-	$indx = $self->{'active'}  if ($indx =~ /^active$/io);
-	$indx2 = $indx  unless (defined $indx2);
-	$indx2 = $#{$self->{'descendants'}}  if (defined($indx2) && $indx2 =~ /^end$/io);
-	return undef  if ($indx < 0 || $indx2 < 0 || $indx > $#{$self->{'descendants'}}
-			|| $indx2 < $indx || $indx2 > $#{$self->{'descendants'}});
+	for (my $i=0;$i<=$#args;$i++) {
+		$args[$i] = $#{$self->{'descendants'}}  if ($args[$i] =~ /end/io);
+		$args[$i] = $self->{'active'}  if ($args[$i] =~ /^active$/io);
+		$args[$i] = $self->index($args[$i])  if ($args[$i] =~ /\D/o);
+	}
+	my @indexRange = (@args);
+	@indexRange = ($args[1] < $args[0]) ? reverse($args[1]..$args[0]) : ($args[0]..$args[1])  if (defined($args[1]) && !defined($args[2]));
 
-	my $fn;
-	for (my $i=$indx; $i<=$indx2; $i++) {
-		$fn = $self->get($i);
+	foreach my $indx (@indexRange) {
+		my $fn = $self->get($indx);
 		${$self->{'selected'}}{$fn} = 0;
-		${$self->{'descendants'}}[$i]->configure(-background => $self->{'btnnormalbg'});
+		${$self->{'descendants'}}[$indx]->configure(-background => $self->{'btnnormalbg'});
 	}
 }
 
@@ -1219,7 +1318,7 @@ the toggle.
 
 =item $thumb->B<selectionClear>(I<index> [ , I<index> ...]);
 
-If a single I<index> is given, that image is "selected".
+If a single I<index> is given, that image is "un-selected".
 If two indices are given, all images between the two, inclusive are 
 de-selected, if selected.  If three or more are given, each image in 
 the list is de-selected.  I<index> can be either a I<number> or I<end>.
